<?php declare(strict_types=1);

namespace EuKit\Base;

class SiteLayout
{
    private const DEFAULT_CONTENT_SIZE_GRID_MAX_COLUMNS = 12;

    private const DEFAULT_CONTENT_SIZE_BREAKPOINTS = [
        'xs' => 0,
        'sm' => 768,
        'md' => 992,
        'lg' => 1200,
    ];

    /**
     * @var SiteLayout
     */
    private static $instance;

    /**
     * @var int|null The maximum width the site's content area can take, in pixels.
     *               This should mimick what is defined in the site's CSS.
     */
    private $maxPageWidth;

    /**
     * @var string|null CSS length, or part of a calc() expression, describing the total horizontal padding & margin
     *                  taken up by a container in the grid. Examples: '10px', '2vw + 1em'
     *                  This should mimick what is defined in the site's CSS.
     */
    private $containerHorizontalPadding;

    /**
     * @var array|null Breakpoints for content size classes.
     *                 [string screenSizeName => int minScreenWidth, ...]
     */
    private $contentSizeBreakpoints;

    /**
     * @var int|null
     */
    private $contentSizeGridMaxColumns;

    /**
     * @var array [string key => [string contentSizeName => int columns, ...]], ...]
     */
    private $contentSizesByKey = [];

    private function __construct()
    {
    }

    public static function getMaxPageWidth(): ?int
    {
        return self::getInstance()->maxPageWidth;
    }

    public static function setMaxPageWidth(?int $width): void
    {
        self::getInstance()->maxPageWidth = $width;
    }

    public static function getContainerHorizontalPadding(): ?string
    {
        return self::getInstance()->containerHorizontalPadding;
    }

    public static function setContainerHorizontalPadding(?string $padding): void
    {
        self::getInstance()->containerHorizontalPadding = $padding;
    }

    public static function getContentSizeGridMaxColumns(): int
    {
        return self::getInstance()->contentSizeGridMaxColumns ?? self::DEFAULT_CONTENT_SIZE_GRID_MAX_COLUMNS;
    }

    public static function setContentSizeGridMaxColumns(?int $columns): void
    {
        self::getInstance()->contentSizeGridMaxColumns = $columns;
    }

    public static function getContentSizeBreakpoints(): array
    {
        return self::getInstance()->contentSizeBreakpoints ?? self::DEFAULT_CONTENT_SIZE_BREAKPOINTS;
    }

    public static function setContentSizeBreakpoint(string $contentSizeName, int $minScreenWidth): void
    {
        self::getInstance()->contentSizeBreakpoints[$contentSizeName] = $minScreenWidth;
    }

    public static function setContentSizeBreakpoints(?array $breakpoints): void
    {
        if ($breakpoints === null) {
            self::getInstance()->contentSizeBreakpoints = null;
            return;
        }

        self::getInstance()->contentSizeBreakpoints = [];
        foreach ($breakpoints as $contentSizeName => $minScreenWidth) {
            self::getInstance()::setContentSizeBreakpoint($contentSizeName, $minScreenWidth);
        }
    }

    /**
     * Associate a set of content sizes to a key, so they can be accessed from several locations.
     *
     * @param string $key
     * @param array  $contentSizes [string contentSizeName => int columns, ...]
     */
    public static function setContentSizes(string $key, array $contentSizes): void
    {
        self::getInstance()->contentSizes[$key] = [];
        foreach ($contentSizes as $contentSizeName => $columns) {
            self::getInstance()->contentSizes[$key][(string)$contentSizeName] = (int)$columns;
        }
    }

    /**
     * @param string $key
     *
     * @return array
     */
    public static function getContentSizes(string $key): array
    {
        return self::getInstance()->contentSizes[$key] ?? [];
    }

    /**
     * @param array $contentSizes [string contentSizeName => int columns, ...]
     *
     * @return string[]
     */
    public static function getContentSizeClasses(array $contentSizes): array
    {
        $breakpoints = array_keys(self::getContentSizeBreakpoints());
        $classes = [];
        foreach ($contentSizes as $contentSizeName => $columns) {
            $classes[] = sprintf('content-size-%s-%s', $contentSizeName, $columns);

            // Automatically add any larger breakpoint size names, that were not mentioned in $contentSizes.
            $foundStartingPoint = false;
            foreach ($breakpoints as $breakpointSizeName) {
                if ($contentSizeName === $breakpointSizeName) {
                    $foundStartingPoint = true;
                    continue;
                }
                if (!$foundStartingPoint) {
                    continue;
                }
                if (!isset($contentSizes[$breakpointSizeName])) {
                    $classes[] = sprintf('content-size-%s-%s', $breakpointSizeName, $columns);
                } else {
                    break;
                }
            }
        }

        return $classes;
    }


    private static function getInstance(): SiteLayout
    {
        if( self::$instance === null ) {
            self::$instance = new self();
        }

        return self::$instance;
    }
}
